# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "InstanceListResponse",
    "CustomMetadata",
    "IndexMethod",
    "IndexingOptions",
    "Metadata",
    "PublicEndpointParams",
    "PublicEndpointParamsChatCompletionsEndpoint",
    "PublicEndpointParamsMcp",
    "PublicEndpointParamsRateLimit",
    "PublicEndpointParamsSearchEndpoint",
    "RetrievalOptions",
    "RetrievalOptionsBoostBy",
    "SourceParams",
    "SourceParamsWebCrawler",
    "SourceParamsWebCrawlerParseOptions",
    "SourceParamsWebCrawlerParseOptionsContentSelector",
]


class CustomMetadata(BaseModel):
    data_type: Literal["text", "number", "boolean", "datetime"]

    field_name: str


class IndexMethod(BaseModel):
    keyword: bool

    vector: bool

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class IndexingOptions(BaseModel):
    keyword_tokenizer: Optional[Literal["porter", "trigram"]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class Metadata(BaseModel):
    created_from_aisearch_wizard: Optional[bool] = None

    worker_domain: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class PublicEndpointParamsChatCompletionsEndpoint(BaseModel):
    disabled: Optional[bool] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class PublicEndpointParamsMcp(BaseModel):
    description: Optional[str] = None

    disabled: Optional[bool] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class PublicEndpointParamsRateLimit(BaseModel):
    period_ms: Optional[int] = None

    requests: Optional[int] = None

    technique: Optional[Literal["fixed", "sliding"]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class PublicEndpointParamsSearchEndpoint(BaseModel):
    disabled: Optional[bool] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class PublicEndpointParams(BaseModel):
    authorized_hosts: Optional[List[str]] = None

    chat_completions_endpoint: Optional[PublicEndpointParamsChatCompletionsEndpoint] = None

    custom_domains: Optional[List[str]] = None

    default_domain_enabled: Optional[bool] = None

    enabled: Optional[bool] = None

    mcp: Optional[PublicEndpointParamsMcp] = None

    rate_limit: Optional[PublicEndpointParamsRateLimit] = None

    search_endpoint: Optional[PublicEndpointParamsSearchEndpoint] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class RetrievalOptionsBoostBy(BaseModel):
    field: str

    data_type: Optional[Literal["number", "datetime", "text", "boolean"]] = FieldInfo(alias="dataType", default=None)

    direction: Optional[Literal["asc", "desc", "exists", "not_exists"]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class RetrievalOptions(BaseModel):
    boost_by: Optional[List[RetrievalOptionsBoostBy]] = None

    keyword_match_mode: Optional[Literal["and", "or"]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class SourceParamsWebCrawlerParseOptionsContentSelector(BaseModel):
    path: str

    selector: str

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class SourceParamsWebCrawlerParseOptions(BaseModel):
    content_selector: Optional[List[SourceParamsWebCrawlerParseOptionsContentSelector]] = None

    include_headers: Optional[Dict[str, str]] = None

    include_images: Optional[bool] = None

    specific_sitemaps: Optional[List[str]] = None

    use_browser_rendering: Optional[bool] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class SourceParamsWebCrawler(BaseModel):
    parse_options: Optional[SourceParamsWebCrawlerParseOptions] = None

    parse_type: Optional[Literal["sitemap", "crawl"]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class SourceParams(BaseModel):
    exclude_items: Optional[List[str]] = None

    include_items: Optional[List[str]] = None

    prefix: Optional[str] = None

    r2_jurisdiction: Optional[str] = None

    web_crawler: Optional[SourceParamsWebCrawler] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class InstanceListResponse(BaseModel):
    id: str

    ai_gateway_id: Optional[str] = None

    aisearch_model: Optional[str] = FieldInfo(alias="ai_search_model", default=None)

    cache: bool

    cache_threshold: Optional[Literal["super_strict_match", "close_enough", "flexible_friend", "anything_goes"]] = None

    cache_ttl: Literal[600, 1800, 3600, 7200, 21600, 43200, 86400, 172800, 259200, 518400]

    chunk: bool

    chunk_overlap: Optional[float] = None

    chunk_size: Optional[float] = None

    created_at: datetime

    created_by: Optional[str] = None

    custom_metadata: Optional[List[CustomMetadata]] = None

    embedding_model: Optional[str] = None

    enable: bool

    engine_version: float

    fusion_method: Literal["max", "rrf"]

    hybrid_search_enabled: bool

    index_method: IndexMethod

    indexing_options: Optional[IndexingOptions] = None

    last_activity: Optional[datetime] = None

    max_num_results: Optional[float] = None

    metadata: Optional[Metadata] = None

    modified_at: datetime

    modified_by: Optional[str] = None

    namespace: str

    paused: bool

    public_endpoint_id: Optional[str] = None

    public_endpoint_params: Optional[PublicEndpointParams] = None

    reranking: bool

    reranking_model: Optional[str] = None

    retrieval_options: Optional[RetrievalOptions] = None

    rewrite_model: Optional[str] = None

    rewrite_query: bool

    score_threshold: Optional[float] = None

    source: Optional[str] = None

    source_params: Optional[SourceParams] = None

    status: str

    summarization: bool

    summarization_model: Optional[str] = None

    sync_interval: Literal[900, 1800, 3600, 7200, 14400, 21600, 43200, 86400]

    system_prompt_aisearch: Optional[str] = FieldInfo(alias="system_prompt_ai_search", default=None)

    system_prompt_index_summarization: Optional[str] = None

    system_prompt_rewrite_query: Optional[str] = None

    token_id: Optional[str] = None

    type: Optional[Literal["r2", "web-crawler"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["Cf1SiteLocationParam"]


class Cf1SiteLocationParam(TypedDict, total=False):
    lat: float
    """Latitude of the CF1 Site."""

    long: float
    """Longitude of the CF1 Site."""

    name: str
    """Name of nearest town, city, or village."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["QueryEditResponse"]


class QueryEditResponse(BaseModel):
    id: int
    """Unique identifier for the saved query"""

    account_id: int
    """Account ID"""

    alert_enabled: bool
    """Whether alerts are enabled"""

    alert_rollup_enabled: bool
    """Whether alert rollup is enabled"""

    created_at: str
    """Creation timestamp"""

    name: str
    """Name of the saved query"""

    query_json: str
    """JSON string containing the query parameters"""

    rule_enabled: bool
    """Whether rule is enabled"""

    updated_at: str
    """Last update timestamp"""

    user_email: str
    """Email of the user who created the query"""

    custom_threat_feed_id: Optional[int] = None
    """Intel Indicator Feed ID (numeric)"""

    rule_list_id: Optional[str] = None
    """WAF rules list ID for blocking"""

    rule_scope: Optional[str] = None
    """Scope for the rule"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["TagEditParams", "Alias", "ExternalReference", "InternalAlias"]


class TagEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    active_duration: Annotated[str, PropertyInfo(alias="activeDuration")]

    actor_category: Annotated[str, PropertyInfo(alias="actorCategory")]
    """Actor variety.

    Allowed values: Activist, Competitor, Customer, Crime Syndicate, Former
    Employee, Nation State, Organized Crime, Nation State Affiliated, Terrorist,
    Unaffiliated.
    """

    actor_category_confidence: Annotated[int, PropertyInfo(alias="actorCategoryConfidence")]
    """Confidence (1-10) in the actor variety (actorCategory).

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    aliases: Iterable[Alias]
    """Structured aliases ({ value, confidence 1-10, tlp }).

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    alias_group_names: Annotated[SequenceNotStr[str], PropertyInfo(alias="aliasGroupNames")]

    alias_group_names_internal: Annotated[SequenceNotStr[str], PropertyInfo(alias="aliasGroupNamesInternal")]

    analytic_priority: Annotated[float, PropertyInfo(alias="analyticPriority")]

    attribution_confidence: Annotated[str, PropertyInfo(alias="attributionConfidence")]

    attribution_confidence_score: Annotated[int, PropertyInfo(alias="attributionConfidenceScore")]

    attribution_organization: Annotated[str, PropertyInfo(alias="attributionOrganization")]

    category_uuid: Annotated[str, PropertyInfo(alias="categoryUuid")]

    date_of_discovery: Annotated[str, PropertyInfo(alias="dateOfDiscovery")]
    """Date the actor was discovered (ISO YYYY-MM-DD)."""

    external_reference_links: Annotated[SequenceNotStr[str], PropertyInfo(alias="externalReferenceLinks")]

    external_references: Annotated[Iterable[ExternalReference], PropertyInfo(alias="externalReferences")]
    """Structured external references ({ url, description }).

    Public: returned to all accounts.
    """

    internal_aliases: Annotated[Iterable[InternalAlias], PropertyInfo(alias="internalAliases")]
    """Internal structured aliases ({ value, confidence 1-10, tlp }).

    CFONE-only: never returned to non-CFONE accounts.
    """

    internal_description: Annotated[str, PropertyInfo(alias="internalDescription")]

    motive: str
    """Actor motive.

    Allowed values: Convenience, Fear, Fun, Financial, Grudge, Ideology, Espionage.
    """

    motive_confidence: Annotated[int, PropertyInfo(alias="motiveConfidence")]
    """Confidence (1-10) in the actor motive.

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    opsec_level: Annotated[str, PropertyInfo(alias="opsecLevel")]

    origin_country_confidence: Annotated[int, PropertyInfo(alias="originCountryConfidence")]
    """Confidence (1-10) in the origin-country attribution.

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    origin_country_iso: Annotated[str, PropertyInfo(alias="originCountryISO")]

    origin_country_tlp: Annotated[Literal["red", "amber", "green", "white"], PropertyInfo(alias="originCountryTlp")]
    """TLP marking for the origin-country attribution.

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    priority: float

    sophistication_level: Annotated[str, PropertyInfo(alias="sophisticationLevel")]

    value: str


class Alias(TypedDict, total=False):
    value: Required[str]

    confidence: Optional[int]

    tlp: Optional[Literal["red", "amber", "green", "white"]]


class ExternalReference(TypedDict, total=False):
    url: Required[str]

    description: Optional[str]


class InternalAlias(TypedDict, total=False):
    value: Required[str]

    confidence: Optional[int]

    tlp: Optional[Literal["red", "amber", "green", "white"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CustomCsrCreateResponse"]


class CustomCsrCreateResponse(BaseModel):
    """A custom Certificate Signing Request (CSR)."""

    id: str
    """Custom CSR identifier tag."""

    created_at: datetime
    """When the CSR was created."""

    key_type: Literal["rsa2048", "p256v1"]
    """The key algorithm used to generate the CSR."""

    account_tag: Optional[str] = None
    """Account identifier associated with this CSR."""

    common_name: Optional[str] = None
    """The common name (domain) for the CSR."""

    country: Optional[str] = None
    """Two-letter ISO 3166-1 alpha-2 country code."""

    csr: Optional[str] = None
    """The PEM-encoded Certificate Signing Request."""

    description: Optional[str] = None
    """Optional description for the CSR."""

    locality: Optional[str] = None
    """City or locality name."""

    name: Optional[str] = None
    """Human-readable name for the CSR."""

    organization: Optional[str] = None
    """Organization name."""

    organizational_unit: Optional[str] = None
    """Organizational unit name."""

    sans: Optional[List[str]] = None
    """Subject Alternative Names included in the CSR."""

    state: Optional[str] = None
    """State or province name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["TagListParams", "Filter"]


class TagListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    cache: Literal["from-graph"]
    """Cache strategy.

    'from-graph' serves results from the graph-node KV cache when all requested
    UUIDs are cached; falls back to normal path on partial/zero hit.
    """

    category_uuid: Annotated[str, PropertyInfo(alias="categoryUuid")]

    filters: Iterable[Filter]
    """Structured filters as a JSON array of {field, op, value} objects.

    Searchable fields: uuid, value, actorCategory, actorCategoryConfidence,
    aliasGroupNames, attributionConfidence, attributionConfidenceScore,
    attributionOrganization, categoryName, motive, motiveConfidence, opsecLevel,
    originCountryISO, originCountryConfidence, sophisticationLevel, priority,
    analyticPriority. Operators: equals, not, contains, startsWith, endsWith, gt,
    lt, gte, lte, like, in, find. Use 'in' for bulk OR within a single field, e.g.
    filters=[{"field":"originCountryISO","op":"in","value":["IR","CN"]}]. Multiple
    entries are AND-joined. Max 10 entries per request, max 100 values per 'in'.
    Per-field notes: `uuid` accepts only 'equals' and 'in' (other operators throw
    ValidationError) — matched against the canonical lowercase storage but callers
    may pass either case (the server lowercases before comparison); index-backed by
    the column's UNIQUE constraint and intended for batched UUID → tag resolution.
    `originCountryISO` uses its B-tree index for equals/not/in. `priority` uses its
    B-tree index for numeric comparisons. Other string columns (`actorCategory`,
    `motive`, etc.) are case-insensitive and unindexed; current catalog size makes
    this a non-issue. `endsWith` and `aliasGroupNames` contains/like are
    leading-wildcard scans and slow on large result sets. `aliasGroupNames` matches
    on the JSON-encoded text, so substrings can cross alias boundaries (a search for
    "apt28" will also match "apt280" if both appear in the same tag's alias list).
    """

    page: float

    page_size: Annotated[float, PropertyInfo(alias="pageSize")]

    search: str
    """Legacy free-text substring match on tag value."""


class Filter(TypedDict, total=False):
    field: Required[
        Literal[
            "uuid",
            "value",
            "actorCategory",
            "actorCategoryConfidence",
            "aliasGroupNames",
            "attributionConfidence",
            "attributionConfidenceScore",
            "attributionOrganization",
            "categoryName",
            "motive",
            "motiveConfidence",
            "opsecLevel",
            "originCountryISO",
            "originCountryConfidence",
            "sophisticationLevel",
            "priority",
            "analyticPriority",
        ]
    ]
    """Tag field to search on.

    Allowed: uuid, value, actorCategory, actorCategoryConfidence, aliasGroupNames,
    attributionConfidence, attributionConfidenceScore, attributionOrganization,
    categoryName, motive, motiveConfidence, opsecLevel, originCountryISO,
    originCountryConfidence, sophisticationLevel, priority, analyticPriority.
    """

    op: Required[
        Literal["equals", "not", "gt", "gte", "lt", "lte", "like", "contains", "startsWith", "endsWith", "in", "find"]
    ]
    """Search operator.

    Use 'in' for bulk OR within a single field, e.g. {field:"originCountryISO",
    op:"in", value:["IR","CN"]}.
    """

    value: Union[str, float, SequenceNotStr[Union[str, float]]]
    """Search value.

    String or number for most operators. Array for 'in' (max 100 items). Country
    values may be passed as alpha-2, alpha-3, name, or common alias (e.g. 'iran',
    'IR', 'IRN') and are normalized to alpha-2 server-side.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .types import (
    TypesResource,
    AsyncTypesResource,
    TypesResourceWithRawResponse,
    AsyncTypesResourceWithRawResponse,
    TypesResourceWithStreamingResponse,
    AsyncTypesResourceWithStreamingResponse,
)
from .aggregate import (
    AggregateResource,
    AsyncAggregateResource,
    AggregateResourceWithRawResponse,
    AsyncAggregateResourceWithRawResponse,
    AggregateResourceWithStreamingResponse,
    AsyncAggregateResourceWithStreamingResponse,
)
from .by_dataset import (
    ByDatasetResource,
    AsyncByDatasetResource,
    ByDatasetResourceWithRawResponse,
    AsyncByDatasetResourceWithRawResponse,
    ByDatasetResourceWithStreamingResponse,
    AsyncByDatasetResourceWithStreamingResponse,
)
from .indicators import (
    IndicatorsResource,
    AsyncIndicatorsResource,
    IndicatorsResourceWithRawResponse,
    AsyncIndicatorsResourceWithRawResponse,
    IndicatorsResourceWithStreamingResponse,
    AsyncIndicatorsResourceWithStreamingResponse,
)

__all__ = [
    "AggregateResource",
    "AsyncAggregateResource",
    "AggregateResourceWithRawResponse",
    "AsyncAggregateResourceWithRawResponse",
    "AggregateResourceWithStreamingResponse",
    "AsyncAggregateResourceWithStreamingResponse",
    "TypesResource",
    "AsyncTypesResource",
    "TypesResourceWithRawResponse",
    "AsyncTypesResourceWithRawResponse",
    "TypesResourceWithStreamingResponse",
    "AsyncTypesResourceWithStreamingResponse",
    "ByDatasetResource",
    "AsyncByDatasetResource",
    "ByDatasetResourceWithRawResponse",
    "AsyncByDatasetResourceWithRawResponse",
    "ByDatasetResourceWithStreamingResponse",
    "AsyncByDatasetResourceWithStreamingResponse",
    "IndicatorsResource",
    "AsyncIndicatorsResource",
    "IndicatorsResourceWithRawResponse",
    "AsyncIndicatorsResourceWithRawResponse",
    "IndicatorsResourceWithStreamingResponse",
    "AsyncIndicatorsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from .data_tags import (
    DataTagsResource,
    AsyncDataTagsResource,
    DataTagsResourceWithRawResponse,
    AsyncDataTagsResourceWithRawResponse,
    DataTagsResourceWithStreamingResponse,
    AsyncDataTagsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.dlp import data_tag_category_create_params, data_tag_category_update_params
from .....types.zero_trust.dlp.data_tag_category_get_response import DataTagCategoryGetResponse
from .....types.zero_trust.dlp.data_tag_category_list_response import DataTagCategoryListResponse
from .....types.zero_trust.dlp.data_tag_category_create_response import DataTagCategoryCreateResponse
from .....types.zero_trust.dlp.data_tag_category_update_response import DataTagCategoryUpdateResponse

__all__ = ["DataTagCategoriesResource", "AsyncDataTagCategoriesResource"]


class DataTagCategoriesResource(SyncAPIResource):
    @cached_property
    def data_tags(self) -> DataTagsResource:
        return DataTagsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DataTagCategoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DataTagCategoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataTagCategoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DataTagCategoriesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        tags: Iterable[data_tag_category_create_params.Tag] | Omit = omit,
        template_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCategoryCreateResponse]:
        """
        Creates a data tag category, optionally from a template.

        Args:
          tags: Tags to create with the category. Mutually exclusive with `template_id`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dlp/data_tag_categories", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "tags": tags,
                    "template_id": template_id,
                },
                data_tag_category_create_params.DataTagCategoryCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCategoryCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCategoryCreateResponse]], ResultWrapper[DataTagCategoryCreateResponse]),
        )

    def update(
        self,
        category_id: str,
        *,
        account_id: str,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        tags: Optional[Iterable[data_tag_category_update_params.Tag]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCategoryUpdateResponse]:
        """
        Updates a data tag category and its tags.

        Args:
          tags: The desired final state of tags.

              - `None` (omitted): no tag changes.
              - `Some([])`: delete all tags.
              - `Some([...])`: desired final set + order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}",
                account_id=account_id,
                category_id=category_id,
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "tags": tags,
                },
                data_tag_category_update_params.DataTagCategoryUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCategoryUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCategoryUpdateResponse]], ResultWrapper[DataTagCategoryUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DataTagCategoryListResponse]:
        """
        Lists data tag categories configured for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/data_tag_categories", account_id=account_id),
            page=SyncSinglePage[DataTagCategoryListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DataTagCategoryListResponse,
        )

    def delete(
        self,
        category_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a data tag category and its tags.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}",
                account_id=account_id,
                category_id=category_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        category_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCategoryGetResponse]:
        """
        Gets a data tag category and its tags.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}",
                account_id=account_id,
                category_id=category_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCategoryGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCategoryGetResponse]], ResultWrapper[DataTagCategoryGetResponse]),
        )


class AsyncDataTagCategoriesResource(AsyncAPIResource):
    @cached_property
    def data_tags(self) -> AsyncDataTagsResource:
        return AsyncDataTagsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDataTagCategoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataTagCategoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataTagCategoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDataTagCategoriesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        tags: Iterable[data_tag_category_create_params.Tag] | Omit = omit,
        template_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCategoryCreateResponse]:
        """
        Creates a data tag category, optionally from a template.

        Args:
          tags: Tags to create with the category. Mutually exclusive with `template_id`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dlp/data_tag_categories", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "tags": tags,
                    "template_id": template_id,
                },
                data_tag_category_create_params.DataTagCategoryCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCategoryCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCategoryCreateResponse]], ResultWrapper[DataTagCategoryCreateResponse]),
        )

    async def update(
        self,
        category_id: str,
        *,
        account_id: str,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        tags: Optional[Iterable[data_tag_category_update_params.Tag]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCategoryUpdateResponse]:
        """
        Updates a data tag category and its tags.

        Args:
          tags: The desired final state of tags.

              - `None` (omitted): no tag changes.
              - `Some([])`: delete all tags.
              - `Some([...])`: desired final set + order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}",
                account_id=account_id,
                category_id=category_id,
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "tags": tags,
                },
                data_tag_category_update_params.DataTagCategoryUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCategoryUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCategoryUpdateResponse]], ResultWrapper[DataTagCategoryUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DataTagCategoryListResponse, AsyncSinglePage[DataTagCategoryListResponse]]:
        """
        Lists data tag categories configured for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/data_tag_categories", account_id=account_id),
            page=AsyncSinglePage[DataTagCategoryListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DataTagCategoryListResponse,
        )

    async def delete(
        self,
        category_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a data tag category and its tags.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}",
                account_id=account_id,
                category_id=category_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        category_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCategoryGetResponse]:
        """
        Gets a data tag category and its tags.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}",
                account_id=account_id,
                category_id=category_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCategoryGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCategoryGetResponse]], ResultWrapper[DataTagCategoryGetResponse]),
        )


class DataTagCategoriesResourceWithRawResponse:
    def __init__(self, data_tag_categories: DataTagCategoriesResource) -> None:
        self._data_tag_categories = data_tag_categories

        self.create = to_raw_response_wrapper(
            data_tag_categories.create,
        )
        self.update = to_raw_response_wrapper(
            data_tag_categories.update,
        )
        self.list = to_raw_response_wrapper(
            data_tag_categories.list,
        )
        self.delete = to_raw_response_wrapper(
            data_tag_categories.delete,
        )
        self.get = to_raw_response_wrapper(
            data_tag_categories.get,
        )

    @cached_property
    def data_tags(self) -> DataTagsResourceWithRawResponse:
        return DataTagsResourceWithRawResponse(self._data_tag_categories.data_tags)


class AsyncDataTagCategoriesResourceWithRawResponse:
    def __init__(self, data_tag_categories: AsyncDataTagCategoriesResource) -> None:
        self._data_tag_categories = data_tag_categories

        self.create = async_to_raw_response_wrapper(
            data_tag_categories.create,
        )
        self.update = async_to_raw_response_wrapper(
            data_tag_categories.update,
        )
        self.list = async_to_raw_response_wrapper(
            data_tag_categories.list,
        )
        self.delete = async_to_raw_response_wrapper(
            data_tag_categories.delete,
        )
        self.get = async_to_raw_response_wrapper(
            data_tag_categories.get,
        )

    @cached_property
    def data_tags(self) -> AsyncDataTagsResourceWithRawResponse:
        return AsyncDataTagsResourceWithRawResponse(self._data_tag_categories.data_tags)


class DataTagCategoriesResourceWithStreamingResponse:
    def __init__(self, data_tag_categories: DataTagCategoriesResource) -> None:
        self._data_tag_categories = data_tag_categories

        self.create = to_streamed_response_wrapper(
            data_tag_categories.create,
        )
        self.update = to_streamed_response_wrapper(
            data_tag_categories.update,
        )
        self.list = to_streamed_response_wrapper(
            data_tag_categories.list,
        )
        self.delete = to_streamed_response_wrapper(
            data_tag_categories.delete,
        )
        self.get = to_streamed_response_wrapper(
            data_tag_categories.get,
        )

    @cached_property
    def data_tags(self) -> DataTagsResourceWithStreamingResponse:
        return DataTagsResourceWithStreamingResponse(self._data_tag_categories.data_tags)


class AsyncDataTagCategoriesResourceWithStreamingResponse:
    def __init__(self, data_tag_categories: AsyncDataTagCategoriesResource) -> None:
        self._data_tag_categories = data_tag_categories

        self.create = async_to_streamed_response_wrapper(
            data_tag_categories.create,
        )
        self.update = async_to_streamed_response_wrapper(
            data_tag_categories.update,
        )
        self.list = async_to_streamed_response_wrapper(
            data_tag_categories.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            data_tag_categories.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            data_tag_categories.get,
        )

    @cached_property
    def data_tags(self) -> AsyncDataTagsResourceWithStreamingResponse:
        return AsyncDataTagsResourceWithStreamingResponse(self._data_tag_categories.data_tags)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["MitigationReviewParams", "Appeal", "Data"]


class MitigationReviewParams(TypedDict, total=False):
    account_id: Required[str]

    appeals: Iterable[Appeal]
    """List of mitigations to appeal."""

    data: Data
    """Counter-notice details supporting an appeal."""

    type: Literal["counter_notice", "content_removed"]
    """The type of appeal being submitted."""


class Appeal(TypedDict, total=False):
    id: Required[str]
    """ID of the mitigation to appeal."""

    reason: Required[Literal["removed", "misclassified"]]
    """Reason why the customer is appealing."""


class Data(TypedDict, total=False):
    """Counter-notice details supporting an appeal."""

    city: Required[str]

    country: Required[str]

    email: Required[str]

    full_name: Required[str]

    jurisdiction_consent: Required[bool]

    perjury_attestation: Required[bool]

    phone_number: Required[str]

    signature: Required[str]

    state: Required[str]

    street_address: Required[str]

    urls: Required[SequenceNotStr[str]]

    zip_code: Required[str]

    company: str

    counter_notice_response: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AggregateListResponse", "Aggregation", "DateRange"]


class Aggregation(BaseModel):
    count: float
    """Number of events for this aggregation"""

    date: Optional[str] = None
    """Date (if groupByDate is true)"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[str]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[str]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[str]]


class DateRange(BaseModel):
    """Date range used for filtering"""

    end_date: Optional[str] = FieldInfo(alias="endDate", default=None)

    start_date: Optional[str] = FieldInfo(alias="startDate", default=None)


class AggregateListResponse(BaseModel):
    aggregate_by: str = FieldInfo(alias="aggregateBy")
    """Column(s) that were aggregated by"""

    aggregations: List[Aggregation]
    """Array of aggregation results with dynamic fields based on aggregateBy columns"""

    total: float
    """Total number of events in the aggregation"""

    date_range: Optional[DateRange] = FieldInfo(alias="dateRange", default=None)
    """Date range used for filtering"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["DatasetEditResponse"]


class DatasetEditResponse(BaseModel):
    is_public: bool = FieldInfo(alias="isPublic")

    name: str

    uuid: str

    deleted_at: Optional[str] = FieldInfo(alias="deletedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["ByDatasetListParams"]


class ByDatasetListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    indicator_type: Annotated[str, PropertyInfo(alias="indicatorType")]

    name: str
    """Filter by indicator value (substring match)"""

    page: float

    page_size: Annotated[float, PropertyInfo(alias="pageSize")]

    related_event: Annotated[SequenceNotStr[str], PropertyInfo(alias="relatedEvent")]
    """Filter indicators by related event UUID(s).

    Multiple UUIDs can be provided by repeating the parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.casb.posture.finding_types import remediation_type_list_params
from ......types.zero_trust.casb.posture.finding_types.remediation_type_list_response import RemediationTypeListResponse

__all__ = ["RemediationTypesResource", "AsyncRemediationTypesResource"]


class RemediationTypesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RemediationTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RemediationTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RemediationTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RemediationTypesResourceWithStreamingResponse(self)

    def list(
        self,
        finding_type_id: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        integration_id: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[RemediationTypeListResponse]:
        """List all remediation types for a given finding type.

        This endpoint supports both
        cursor and offset pagination. Note that `cursor` and `page` are mutually
        exclusive.

        Args:
          cursor: A cursor for pagination.

          integration_id: Filter by an integration ID

          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_type_id:
            raise ValueError(f"Expected a non-empty value for `finding_type_id` but received {finding_type_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/data-security/posture/finding_types/{finding_type_id}/remediation_types",
                account_id=account_id,
                finding_type_id=finding_type_id,
            ),
            page=SyncV4PagePaginationArray[RemediationTypeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "integration_id": integration_id,
                        "page": page,
                        "per_page": per_page,
                    },
                    remediation_type_list_params.RemediationTypeListParams,
                ),
            ),
            model=RemediationTypeListResponse,
        )


class AsyncRemediationTypesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRemediationTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRemediationTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRemediationTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRemediationTypesResourceWithStreamingResponse(self)

    def list(
        self,
        finding_type_id: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        integration_id: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RemediationTypeListResponse, AsyncV4PagePaginationArray[RemediationTypeListResponse]]:
        """List all remediation types for a given finding type.

        This endpoint supports both
        cursor and offset pagination. Note that `cursor` and `page` are mutually
        exclusive.

        Args:
          cursor: A cursor for pagination.

          integration_id: Filter by an integration ID

          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_type_id:
            raise ValueError(f"Expected a non-empty value for `finding_type_id` but received {finding_type_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/data-security/posture/finding_types/{finding_type_id}/remediation_types",
                account_id=account_id,
                finding_type_id=finding_type_id,
            ),
            page=AsyncV4PagePaginationArray[RemediationTypeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "integration_id": integration_id,
                        "page": page,
                        "per_page": per_page,
                    },
                    remediation_type_list_params.RemediationTypeListParams,
                ),
            ),
            model=RemediationTypeListResponse,
        )


class RemediationTypesResourceWithRawResponse:
    def __init__(self, remediation_types: RemediationTypesResource) -> None:
        self._remediation_types = remediation_types

        self.list = to_raw_response_wrapper(
            remediation_types.list,
        )


class AsyncRemediationTypesResourceWithRawResponse:
    def __init__(self, remediation_types: AsyncRemediationTypesResource) -> None:
        self._remediation_types = remediation_types

        self.list = async_to_raw_response_wrapper(
            remediation_types.list,
        )


class RemediationTypesResourceWithStreamingResponse:
    def __init__(self, remediation_types: RemediationTypesResource) -> None:
        self._remediation_types = remediation_types

        self.list = to_streamed_response_wrapper(
            remediation_types.list,
        )


class AsyncRemediationTypesResourceWithStreamingResponse:
    def __init__(self, remediation_types: AsyncRemediationTypesResource) -> None:
        self._remediation_types = remediation_types

        self.list = async_to_streamed_response_wrapper(
            remediation_types.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .indicators.indicators import (
    IndicatorsResource,
    AsyncIndicatorsResource,
    IndicatorsResourceWithRawResponse,
    AsyncIndicatorsResourceWithRawResponse,
    IndicatorsResourceWithStreamingResponse,
    AsyncIndicatorsResourceWithStreamingResponse,
)
from .....types.cloudforce_one.threat_events import tag_edit_params, tag_list_params, tag_create_params
from .....types.cloudforce_one.threat_events.tag_edit_response import TagEditResponse
from .....types.cloudforce_one.threat_events.tag_list_response import TagListResponse
from .....types.cloudforce_one.threat_events.tag_create_response import TagCreateResponse
from .....types.cloudforce_one.threat_events.tag_delete_response import TagDeleteResponse

__all__ = ["TagsResource", "AsyncTagsResource"]


class TagsResource(SyncAPIResource):
    @cached_property
    def categories(self) -> CategoriesResource:
        return CategoriesResource(self._client)

    @cached_property
    def indicators(self) -> IndicatorsResource:
        return IndicatorsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TagsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        value: str,
        active_duration: str | Omit = omit,
        actor_category: str | Omit = omit,
        actor_category_confidence: int | Omit = omit,
        aliases: Iterable[tag_create_params.Alias] | Omit = omit,
        alias_group_names: SequenceNotStr[str] | Omit = omit,
        alias_group_names_internal: SequenceNotStr[str] | Omit = omit,
        analytic_priority: float | Omit = omit,
        attribution_confidence: str | Omit = omit,
        attribution_confidence_score: int | Omit = omit,
        attribution_organization: str | Omit = omit,
        category_uuid: str | Omit = omit,
        date_of_discovery: str | Omit = omit,
        external_reference_links: SequenceNotStr[str] | Omit = omit,
        external_references: Iterable[tag_create_params.ExternalReference] | Omit = omit,
        internal_aliases: Iterable[tag_create_params.InternalAlias] | Omit = omit,
        internal_description: str | Omit = omit,
        motive: str | Omit = omit,
        motive_confidence: int | Omit = omit,
        opsec_level: str | Omit = omit,
        origin_country_confidence: int | Omit = omit,
        origin_country_iso: str | Omit = omit,
        origin_country_tlp: Literal["red", "amber", "green", "white"] | Omit = omit,
        priority: float | Omit = omit,
        sophistication_level: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagCreateResponse:
        """
        Creates a new tag to be used accross threat events.

        Args:
          account_id: Account ID.

          actor_category: Actor variety. Allowed values: Activist, Competitor, Customer, Crime Syndicate,
              Former Employee, Nation State, Organized Crime, Nation State Affiliated,
              Terrorist, Unaffiliated.

          actor_category_confidence: Confidence (1-10) in the actor variety (actorCategory). CFONE-only: stripped
              from responses to non-CFONE accounts.

          aliases: Structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: stripped from
              responses to non-CFONE accounts.

          date_of_discovery: Date the actor was discovered (ISO YYYY-MM-DD).

          external_references: Structured external references ({ url, description }). Public: returned to all
              accounts.

          internal_aliases: Internal structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: never
              returned to non-CFONE accounts.

          motive: Actor motive. Allowed values: Convenience, Fear, Fun, Financial, Grudge,
              Ideology, Espionage.

          motive_confidence: Confidence (1-10) in the actor motive. CFONE-only: stripped from responses to
              non-CFONE accounts.

          origin_country_confidence: Confidence (1-10) in the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          origin_country_tlp: TLP marking for the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/cloudforce-one/events/tags/create", account_id=account_id),
            body=maybe_transform(
                {
                    "value": value,
                    "active_duration": active_duration,
                    "actor_category": actor_category,
                    "actor_category_confidence": actor_category_confidence,
                    "aliases": aliases,
                    "alias_group_names": alias_group_names,
                    "alias_group_names_internal": alias_group_names_internal,
                    "analytic_priority": analytic_priority,
                    "attribution_confidence": attribution_confidence,
                    "attribution_confidence_score": attribution_confidence_score,
                    "attribution_organization": attribution_organization,
                    "category_uuid": category_uuid,
                    "date_of_discovery": date_of_discovery,
                    "external_reference_links": external_reference_links,
                    "external_references": external_references,
                    "internal_aliases": internal_aliases,
                    "internal_description": internal_description,
                    "motive": motive,
                    "motive_confidence": motive_confidence,
                    "opsec_level": opsec_level,
                    "origin_country_confidence": origin_country_confidence,
                    "origin_country_iso": origin_country_iso,
                    "origin_country_tlp": origin_country_tlp,
                    "priority": priority,
                    "sophistication_level": sophistication_level,
                },
                tag_create_params.TagCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagCreateResponse,
        )

    def list(
        self,
        *,
        account_id: str,
        cache: Literal["from-graph"] | Omit = omit,
        category_uuid: str | Omit = omit,
        filters: Iterable[tag_list_params.Filter] | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagListResponse:
        """Returns all Source-of-Truth tags for an account.

        Supports legacy free-text
        `search` on tag value and `categoryUuid` exact match, plus a structured
        `filters` JSON array for filtering by metadata fields (originCountryISO,
        actorCategory, motive, priority, etc.). Country values may be passed as alpha-2,
        alpha-3, name, or common alias.

        Args:
          account_id: Account ID.

          cache: Cache strategy. 'from-graph' serves results from the graph-node KV cache when
              all requested UUIDs are cached; falls back to normal path on partial/zero hit.

          filters: Structured filters as a JSON array of {field, op, value} objects. Searchable
              fields: uuid, value, actorCategory, actorCategoryConfidence, aliasGroupNames,
              attributionConfidence, attributionConfidenceScore, attributionOrganization,
              categoryName, motive, motiveConfidence, opsecLevel, originCountryISO,
              originCountryConfidence, sophisticationLevel, priority, analyticPriority.
              Operators: equals, not, contains, startsWith, endsWith, gt, lt, gte, lte, like,
              in, find. Use 'in' for bulk OR within a single field, e.g.
              filters=[{"field":"originCountryISO","op":"in","value":["IR","CN"]}]. Multiple
              entries are AND-joined. Max 10 entries per request, max 100 values per 'in'.
              Per-field notes: `uuid` accepts only 'equals' and 'in' (other operators throw
              ValidationError) — matched against the canonical lowercase storage but callers
              may pass either case (the server lowercases before comparison); index-backed by
              the column's UNIQUE constraint and intended for batched UUID → tag resolution.
              `originCountryISO` uses its B-tree index for equals/not/in. `priority` uses its
              B-tree index for numeric comparisons. Other string columns (`actorCategory`,
              `motive`, etc.) are case-insensitive and unindexed; current catalog size makes
              this a non-issue. `endsWith` and `aliasGroupNames` contains/like are
              leading-wildcard scans and slow on large result sets. `aliasGroupNames` matches
              on the JSON-encoded text, so substrings can cross alias boundaries (a search for
              "apt28" will also match "apt280" if both appear in the same tag's alias list).

          search: Legacy free-text substring match on tag value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/tags", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cache": cache,
                        "category_uuid": category_uuid,
                        "filters": filters,
                        "page": page,
                        "page_size": page_size,
                        "search": search,
                    },
                    tag_list_params.TagListParams,
                ),
            ),
            cast_to=TagListResponse,
        )

    def delete(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagDeleteResponse:
        """
        Deletes a Source-of-Truth tag by UUID.

        Args:
          account_id: Account ID.

          tag_uuid: Tag UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}", account_id=account_id, tag_uuid=tag_uuid
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagDeleteResponse,
        )

    def edit(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        active_duration: str | Omit = omit,
        actor_category: str | Omit = omit,
        actor_category_confidence: int | Omit = omit,
        aliases: Iterable[tag_edit_params.Alias] | Omit = omit,
        alias_group_names: SequenceNotStr[str] | Omit = omit,
        alias_group_names_internal: SequenceNotStr[str] | Omit = omit,
        analytic_priority: float | Omit = omit,
        attribution_confidence: str | Omit = omit,
        attribution_confidence_score: int | Omit = omit,
        attribution_organization: str | Omit = omit,
        category_uuid: str | Omit = omit,
        date_of_discovery: str | Omit = omit,
        external_reference_links: SequenceNotStr[str] | Omit = omit,
        external_references: Iterable[tag_edit_params.ExternalReference] | Omit = omit,
        internal_aliases: Iterable[tag_edit_params.InternalAlias] | Omit = omit,
        internal_description: str | Omit = omit,
        motive: str | Omit = omit,
        motive_confidence: int | Omit = omit,
        opsec_level: str | Omit = omit,
        origin_country_confidence: int | Omit = omit,
        origin_country_iso: str | Omit = omit,
        origin_country_tlp: Literal["red", "amber", "green", "white"] | Omit = omit,
        priority: float | Omit = omit,
        sophistication_level: str | Omit = omit,
        value: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagEditResponse:
        """
        Updates a Source-of-Truth tag by UUID.

        Args:
          account_id: Account ID.

          tag_uuid: Tag UUID.

          actor_category: Actor variety. Allowed values: Activist, Competitor, Customer, Crime Syndicate,
              Former Employee, Nation State, Organized Crime, Nation State Affiliated,
              Terrorist, Unaffiliated.

          actor_category_confidence: Confidence (1-10) in the actor variety (actorCategory). CFONE-only: stripped
              from responses to non-CFONE accounts.

          aliases: Structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: stripped from
              responses to non-CFONE accounts.

          date_of_discovery: Date the actor was discovered (ISO YYYY-MM-DD).

          external_references: Structured external references ({ url, description }). Public: returned to all
              accounts.

          internal_aliases: Internal structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: never
              returned to non-CFONE accounts.

          motive: Actor motive. Allowed values: Convenience, Fear, Fun, Financial, Grudge,
              Ideology, Espionage.

          motive_confidence: Confidence (1-10) in the actor motive. CFONE-only: stripped from responses to
              non-CFONE accounts.

          origin_country_confidence: Confidence (1-10) in the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          origin_country_tlp: TLP marking for the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}", account_id=account_id, tag_uuid=tag_uuid
            ),
            body=maybe_transform(
                {
                    "active_duration": active_duration,
                    "actor_category": actor_category,
                    "actor_category_confidence": actor_category_confidence,
                    "aliases": aliases,
                    "alias_group_names": alias_group_names,
                    "alias_group_names_internal": alias_group_names_internal,
                    "analytic_priority": analytic_priority,
                    "attribution_confidence": attribution_confidence,
                    "attribution_confidence_score": attribution_confidence_score,
                    "attribution_organization": attribution_organization,
                    "category_uuid": category_uuid,
                    "date_of_discovery": date_of_discovery,
                    "external_reference_links": external_reference_links,
                    "external_references": external_references,
                    "internal_aliases": internal_aliases,
                    "internal_description": internal_description,
                    "motive": motive,
                    "motive_confidence": motive_confidence,
                    "opsec_level": opsec_level,
                    "origin_country_confidence": origin_country_confidence,
                    "origin_country_iso": origin_country_iso,
                    "origin_country_tlp": origin_country_tlp,
                    "priority": priority,
                    "sophistication_level": sophistication_level,
                    "value": value,
                },
                tag_edit_params.TagEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagEditResponse,
        )


class AsyncTagsResource(AsyncAPIResource):
    @cached_property
    def categories(self) -> AsyncCategoriesResource:
        return AsyncCategoriesResource(self._client)

    @cached_property
    def indicators(self) -> AsyncIndicatorsResource:
        return AsyncIndicatorsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTagsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        value: str,
        active_duration: str | Omit = omit,
        actor_category: str | Omit = omit,
        actor_category_confidence: int | Omit = omit,
        aliases: Iterable[tag_create_params.Alias] | Omit = omit,
        alias_group_names: SequenceNotStr[str] | Omit = omit,
        alias_group_names_internal: SequenceNotStr[str] | Omit = omit,
        analytic_priority: float | Omit = omit,
        attribution_confidence: str | Omit = omit,
        attribution_confidence_score: int | Omit = omit,
        attribution_organization: str | Omit = omit,
        category_uuid: str | Omit = omit,
        date_of_discovery: str | Omit = omit,
        external_reference_links: SequenceNotStr[str] | Omit = omit,
        external_references: Iterable[tag_create_params.ExternalReference] | Omit = omit,
        internal_aliases: Iterable[tag_create_params.InternalAlias] | Omit = omit,
        internal_description: str | Omit = omit,
        motive: str | Omit = omit,
        motive_confidence: int | Omit = omit,
        opsec_level: str | Omit = omit,
        origin_country_confidence: int | Omit = omit,
        origin_country_iso: str | Omit = omit,
        origin_country_tlp: Literal["red", "amber", "green", "white"] | Omit = omit,
        priority: float | Omit = omit,
        sophistication_level: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagCreateResponse:
        """
        Creates a new tag to be used accross threat events.

        Args:
          account_id: Account ID.

          actor_category: Actor variety. Allowed values: Activist, Competitor, Customer, Crime Syndicate,
              Former Employee, Nation State, Organized Crime, Nation State Affiliated,
              Terrorist, Unaffiliated.

          actor_category_confidence: Confidence (1-10) in the actor variety (actorCategory). CFONE-only: stripped
              from responses to non-CFONE accounts.

          aliases: Structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: stripped from
              responses to non-CFONE accounts.

          date_of_discovery: Date the actor was discovered (ISO YYYY-MM-DD).

          external_references: Structured external references ({ url, description }). Public: returned to all
              accounts.

          internal_aliases: Internal structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: never
              returned to non-CFONE accounts.

          motive: Actor motive. Allowed values: Convenience, Fear, Fun, Financial, Grudge,
              Ideology, Espionage.

          motive_confidence: Confidence (1-10) in the actor motive. CFONE-only: stripped from responses to
              non-CFONE accounts.

          origin_country_confidence: Confidence (1-10) in the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          origin_country_tlp: TLP marking for the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/cloudforce-one/events/tags/create", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "value": value,
                    "active_duration": active_duration,
                    "actor_category": actor_category,
                    "actor_category_confidence": actor_category_confidence,
                    "aliases": aliases,
                    "alias_group_names": alias_group_names,
                    "alias_group_names_internal": alias_group_names_internal,
                    "analytic_priority": analytic_priority,
                    "attribution_confidence": attribution_confidence,
                    "attribution_confidence_score": attribution_confidence_score,
                    "attribution_organization": attribution_organization,
                    "category_uuid": category_uuid,
                    "date_of_discovery": date_of_discovery,
                    "external_reference_links": external_reference_links,
                    "external_references": external_references,
                    "internal_aliases": internal_aliases,
                    "internal_description": internal_description,
                    "motive": motive,
                    "motive_confidence": motive_confidence,
                    "opsec_level": opsec_level,
                    "origin_country_confidence": origin_country_confidence,
                    "origin_country_iso": origin_country_iso,
                    "origin_country_tlp": origin_country_tlp,
                    "priority": priority,
                    "sophistication_level": sophistication_level,
                },
                tag_create_params.TagCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagCreateResponse,
        )

    async def list(
        self,
        *,
        account_id: str,
        cache: Literal["from-graph"] | Omit = omit,
        category_uuid: str | Omit = omit,
        filters: Iterable[tag_list_params.Filter] | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagListResponse:
        """Returns all Source-of-Truth tags for an account.

        Supports legacy free-text
        `search` on tag value and `categoryUuid` exact match, plus a structured
        `filters` JSON array for filtering by metadata fields (originCountryISO,
        actorCategory, motive, priority, etc.). Country values may be passed as alpha-2,
        alpha-3, name, or common alias.

        Args:
          account_id: Account ID.

          cache: Cache strategy. 'from-graph' serves results from the graph-node KV cache when
              all requested UUIDs are cached; falls back to normal path on partial/zero hit.

          filters: Structured filters as a JSON array of {field, op, value} objects. Searchable
              fields: uuid, value, actorCategory, actorCategoryConfidence, aliasGroupNames,
              attributionConfidence, attributionConfidenceScore, attributionOrganization,
              categoryName, motive, motiveConfidence, opsecLevel, originCountryISO,
              originCountryConfidence, sophisticationLevel, priority, analyticPriority.
              Operators: equals, not, contains, startsWith, endsWith, gt, lt, gte, lte, like,
              in, find. Use 'in' for bulk OR within a single field, e.g.
              filters=[{"field":"originCountryISO","op":"in","value":["IR","CN"]}]. Multiple
              entries are AND-joined. Max 10 entries per request, max 100 values per 'in'.
              Per-field notes: `uuid` accepts only 'equals' and 'in' (other operators throw
              ValidationError) — matched against the canonical lowercase storage but callers
              may pass either case (the server lowercases before comparison); index-backed by
              the column's UNIQUE constraint and intended for batched UUID → tag resolution.
              `originCountryISO` uses its B-tree index for equals/not/in. `priority` uses its
              B-tree index for numeric comparisons. Other string columns (`actorCategory`,
              `motive`, etc.) are case-insensitive and unindexed; current catalog size makes
              this a non-issue. `endsWith` and `aliasGroupNames` contains/like are
              leading-wildcard scans and slow on large result sets. `aliasGroupNames` matches
              on the JSON-encoded text, so substrings can cross alias boundaries (a search for
              "apt28" will also match "apt280" if both appear in the same tag's alias list).

          search: Legacy free-text substring match on tag value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/tags", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cache": cache,
                        "category_uuid": category_uuid,
                        "filters": filters,
                        "page": page,
                        "page_size": page_size,
                        "search": search,
                    },
                    tag_list_params.TagListParams,
                ),
            ),
            cast_to=TagListResponse,
        )

    async def delete(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagDeleteResponse:
        """
        Deletes a Source-of-Truth tag by UUID.

        Args:
          account_id: Account ID.

          tag_uuid: Tag UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}", account_id=account_id, tag_uuid=tag_uuid
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagDeleteResponse,
        )

    async def edit(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        active_duration: str | Omit = omit,
        actor_category: str | Omit = omit,
        actor_category_confidence: int | Omit = omit,
        aliases: Iterable[tag_edit_params.Alias] | Omit = omit,
        alias_group_names: SequenceNotStr[str] | Omit = omit,
        alias_group_names_internal: SequenceNotStr[str] | Omit = omit,
        analytic_priority: float | Omit = omit,
        attribution_confidence: str | Omit = omit,
        attribution_confidence_score: int | Omit = omit,
        attribution_organization: str | Omit = omit,
        category_uuid: str | Omit = omit,
        date_of_discovery: str | Omit = omit,
        external_reference_links: SequenceNotStr[str] | Omit = omit,
        external_references: Iterable[tag_edit_params.ExternalReference] | Omit = omit,
        internal_aliases: Iterable[tag_edit_params.InternalAlias] | Omit = omit,
        internal_description: str | Omit = omit,
        motive: str | Omit = omit,
        motive_confidence: int | Omit = omit,
        opsec_level: str | Omit = omit,
        origin_country_confidence: int | Omit = omit,
        origin_country_iso: str | Omit = omit,
        origin_country_tlp: Literal["red", "amber", "green", "white"] | Omit = omit,
        priority: float | Omit = omit,
        sophistication_level: str | Omit = omit,
        value: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagEditResponse:
        """
        Updates a Source-of-Truth tag by UUID.

        Args:
          account_id: Account ID.

          tag_uuid: Tag UUID.

          actor_category: Actor variety. Allowed values: Activist, Competitor, Customer, Crime Syndicate,
              Former Employee, Nation State, Organized Crime, Nation State Affiliated,
              Terrorist, Unaffiliated.

          actor_category_confidence: Confidence (1-10) in the actor variety (actorCategory). CFONE-only: stripped
              from responses to non-CFONE accounts.

          aliases: Structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: stripped from
              responses to non-CFONE accounts.

          date_of_discovery: Date the actor was discovered (ISO YYYY-MM-DD).

          external_references: Structured external references ({ url, description }). Public: returned to all
              accounts.

          internal_aliases: Internal structured aliases ({ value, confidence 1-10, tlp }). CFONE-only: never
              returned to non-CFONE accounts.

          motive: Actor motive. Allowed values: Convenience, Fear, Fun, Financial, Grudge,
              Ideology, Espionage.

          motive_confidence: Confidence (1-10) in the actor motive. CFONE-only: stripped from responses to
              non-CFONE accounts.

          origin_country_confidence: Confidence (1-10) in the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          origin_country_tlp: TLP marking for the origin-country attribution. CFONE-only: stripped from
              responses to non-CFONE accounts.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}", account_id=account_id, tag_uuid=tag_uuid
            ),
            body=await async_maybe_transform(
                {
                    "active_duration": active_duration,
                    "actor_category": actor_category,
                    "actor_category_confidence": actor_category_confidence,
                    "aliases": aliases,
                    "alias_group_names": alias_group_names,
                    "alias_group_names_internal": alias_group_names_internal,
                    "analytic_priority": analytic_priority,
                    "attribution_confidence": attribution_confidence,
                    "attribution_confidence_score": attribution_confidence_score,
                    "attribution_organization": attribution_organization,
                    "category_uuid": category_uuid,
                    "date_of_discovery": date_of_discovery,
                    "external_reference_links": external_reference_links,
                    "external_references": external_references,
                    "internal_aliases": internal_aliases,
                    "internal_description": internal_description,
                    "motive": motive,
                    "motive_confidence": motive_confidence,
                    "opsec_level": opsec_level,
                    "origin_country_confidence": origin_country_confidence,
                    "origin_country_iso": origin_country_iso,
                    "origin_country_tlp": origin_country_tlp,
                    "priority": priority,
                    "sophistication_level": sophistication_level,
                    "value": value,
                },
                tag_edit_params.TagEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagEditResponse,
        )


class TagsResourceWithRawResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.create = to_raw_response_wrapper(
            tags.create,
        )
        self.list = to_raw_response_wrapper(
            tags.list,
        )
        self.delete = to_raw_response_wrapper(
            tags.delete,
        )
        self.edit = to_raw_response_wrapper(
            tags.edit,
        )

    @cached_property
    def categories(self) -> CategoriesResourceWithRawResponse:
        return CategoriesResourceWithRawResponse(self._tags.categories)

    @cached_property
    def indicators(self) -> IndicatorsResourceWithRawResponse:
        return IndicatorsResourceWithRawResponse(self._tags.indicators)


class AsyncTagsResourceWithRawResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.create = async_to_raw_response_wrapper(
            tags.create,
        )
        self.list = async_to_raw_response_wrapper(
            tags.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tags.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            tags.edit,
        )

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithRawResponse:
        return AsyncCategoriesResourceWithRawResponse(self._tags.categories)

    @cached_property
    def indicators(self) -> AsyncIndicatorsResourceWithRawResponse:
        return AsyncIndicatorsResourceWithRawResponse(self._tags.indicators)


class TagsResourceWithStreamingResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.create = to_streamed_response_wrapper(
            tags.create,
        )
        self.list = to_streamed_response_wrapper(
            tags.list,
        )
        self.delete = to_streamed_response_wrapper(
            tags.delete,
        )
        self.edit = to_streamed_response_wrapper(
            tags.edit,
        )

    @cached_property
    def categories(self) -> CategoriesResourceWithStreamingResponse:
        return CategoriesResourceWithStreamingResponse(self._tags.categories)

    @cached_property
    def indicators(self) -> IndicatorsResourceWithStreamingResponse:
        return IndicatorsResourceWithStreamingResponse(self._tags.indicators)


class AsyncTagsResourceWithStreamingResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.create = async_to_streamed_response_wrapper(
            tags.create,
        )
        self.list = async_to_streamed_response_wrapper(
            tags.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tags.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            tags.edit,
        )

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithStreamingResponse:
        return AsyncCategoriesResourceWithStreamingResponse(self._tags.categories)

    @cached_property
    def indicators(self) -> AsyncIndicatorsResourceWithStreamingResponse:
        return AsyncIndicatorsResourceWithStreamingResponse(self._tags.indicators)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Iterable

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_base_client import make_request_options
from ......types.cloudforce_one.threat_events.tags.indicators import by_dataset_list_params
from ......types.cloudforce_one.threat_events.tags.indicators.by_dataset_list_response import ByDatasetListResponse

__all__ = ["ByDatasetResource", "AsyncByDatasetResource"]


class ByDatasetResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ByDatasetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ByDatasetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ByDatasetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ByDatasetResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use list instead (GET /accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}/indicators)."
    )
    def list(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        dataset_id: str,
        indicator_type: str | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        related_event: SequenceNotStr[str] | Omit = omit,
        search: Iterable[by_dataset_list_params.Search] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ByDatasetListResponse:
        """This endpoint is deprecated.

        Use GET
        /:account_id/events/tags/:tag_uuid/indicators with the optional datasetIds query
        parameter instead. Returns indicators associated with the provided tag UUID
        within a single dataset's indicator shards, with pagination.

        Args:
          account_id: Account ID.

          dataset_id: Dataset UUID.

          tag_uuid: Tag UUID.

          related_event: Filter indicators by related event UUID(s). Multiple UUIDs can be provided by
              repeating the parameter.

          search: Structured search as a JSON array of {field, op, value} objects. Searchable
              fields: value, indicatorType. Multiple conditions are AND'd together. Max 10
              conditions per request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}/tags/{tag_uuid}/indicators",
                account_id=account_id,
                dataset_id=dataset_id,
                tag_uuid=tag_uuid,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "indicator_type": indicator_type,
                        "page": page,
                        "page_size": page_size,
                        "related_event": related_event,
                        "search": search,
                    },
                    by_dataset_list_params.ByDatasetListParams,
                ),
            ),
            cast_to=ByDatasetListResponse,
        )


class AsyncByDatasetResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncByDatasetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncByDatasetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncByDatasetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncByDatasetResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use list instead (GET /accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}/indicators)."
    )
    async def list(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        dataset_id: str,
        indicator_type: str | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        related_event: SequenceNotStr[str] | Omit = omit,
        search: Iterable[by_dataset_list_params.Search] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ByDatasetListResponse:
        """This endpoint is deprecated.

        Use GET
        /:account_id/events/tags/:tag_uuid/indicators with the optional datasetIds query
        parameter instead. Returns indicators associated with the provided tag UUID
        within a single dataset's indicator shards, with pagination.

        Args:
          account_id: Account ID.

          dataset_id: Dataset UUID.

          tag_uuid: Tag UUID.

          related_event: Filter indicators by related event UUID(s). Multiple UUIDs can be provided by
              repeating the parameter.

          search: Structured search as a JSON array of {field, op, value} objects. Searchable
              fields: value, indicatorType. Multiple conditions are AND'd together. Max 10
              conditions per request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}/tags/{tag_uuid}/indicators",
                account_id=account_id,
                dataset_id=dataset_id,
                tag_uuid=tag_uuid,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "indicator_type": indicator_type,
                        "page": page,
                        "page_size": page_size,
                        "related_event": related_event,
                        "search": search,
                    },
                    by_dataset_list_params.ByDatasetListParams,
                ),
            ),
            cast_to=ByDatasetListResponse,
        )


class ByDatasetResourceWithRawResponse:
    def __init__(self, by_dataset: ByDatasetResource) -> None:
        self._by_dataset = by_dataset

        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                by_dataset.list,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncByDatasetResourceWithRawResponse:
    def __init__(self, by_dataset: AsyncByDatasetResource) -> None:
        self._by_dataset = by_dataset

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                by_dataset.list,  # pyright: ignore[reportDeprecated],
            )
        )


class ByDatasetResourceWithStreamingResponse:
    def __init__(self, by_dataset: ByDatasetResource) -> None:
        self._by_dataset = by_dataset

        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                by_dataset.list,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncByDatasetResourceWithStreamingResponse:
    def __init__(self, by_dataset: AsyncByDatasetResource) -> None:
        self._by_dataset = by_dataset

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                by_dataset.list,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .apps import (
    AppsResource,
    AsyncAppsResource,
    AppsResourceWithRawResponse,
    AsyncAppsResourceWithRawResponse,
    AppsResourceWithStreamingResponse,
    AsyncAppsResourceWithStreamingResponse,
)
from .flagship import (
    FlagshipResource,
    AsyncFlagshipResource,
    FlagshipResourceWithRawResponse,
    AsyncFlagshipResourceWithRawResponse,
    FlagshipResourceWithStreamingResponse,
    AsyncFlagshipResourceWithStreamingResponse,
)

__all__ = [
    "AppsResource",
    "AsyncAppsResource",
    "AppsResourceWithRawResponse",
    "AsyncAppsResourceWithRawResponse",
    "AppsResourceWithStreamingResponse",
    "AsyncAppsResourceWithStreamingResponse",
    "FlagshipResource",
    "AsyncFlagshipResource",
    "FlagshipResourceWithRawResponse",
    "AsyncFlagshipResourceWithRawResponse",
    "FlagshipResourceWithStreamingResponse",
    "AsyncFlagshipResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from .remediation_types import (
    RemediationTypesResource,
    AsyncRemediationTypesResource,
    RemediationTypesResourceWithRawResponse,
    AsyncRemediationTypesResourceWithRawResponse,
    RemediationTypesResourceWithStreamingResponse,
    AsyncRemediationTypesResourceWithStreamingResponse,
)
from ......types.zero_trust.casb.posture import finding_type_list_params
from ......types.zero_trust.casb.posture.finding_type_get_response import FindingTypeGetResponse
from ......types.zero_trust.casb.posture.finding_type_list_response import FindingTypeListResponse

__all__ = ["FindingTypesResource", "AsyncFindingTypesResource"]


class FindingTypesResource(SyncAPIResource):
    @cached_property
    def remediation_types(self) -> RemediationTypesResource:
        return RemediationTypesResource(self._client)

    @cached_property
    def with_raw_response(self) -> FindingTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FindingTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FindingTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FindingTypesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[FindingTypeListResponse]:
        """
        List all available finding types with pagination support.

        Args:
          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/data-security/posture/finding_types", account_id=account_id),
            page=SyncV4PagePaginationArray[FindingTypeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    finding_type_list_params.FindingTypeListParams,
                ),
            ),
            model=FindingTypeListResponse,
        )

    def get(
        self,
        finding_type_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingTypeGetResponse]:
        """
        Retrieve a specific finding type by its unique identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_type_id:
            raise ValueError(f"Expected a non-empty value for `finding_type_id` but received {finding_type_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/data-security/posture/finding_types/{finding_type_id}",
                account_id=account_id,
                finding_type_id=finding_type_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingTypeGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingTypeGetResponse]], ResultWrapper[FindingTypeGetResponse]),
        )


class AsyncFindingTypesResource(AsyncAPIResource):
    @cached_property
    def remediation_types(self) -> AsyncRemediationTypesResource:
        return AsyncRemediationTypesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFindingTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFindingTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFindingTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFindingTypesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FindingTypeListResponse, AsyncV4PagePaginationArray[FindingTypeListResponse]]:
        """
        List all available finding types with pagination support.

        Args:
          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/data-security/posture/finding_types", account_id=account_id),
            page=AsyncV4PagePaginationArray[FindingTypeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    finding_type_list_params.FindingTypeListParams,
                ),
            ),
            model=FindingTypeListResponse,
        )

    async def get(
        self,
        finding_type_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingTypeGetResponse]:
        """
        Retrieve a specific finding type by its unique identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_type_id:
            raise ValueError(f"Expected a non-empty value for `finding_type_id` but received {finding_type_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/data-security/posture/finding_types/{finding_type_id}",
                account_id=account_id,
                finding_type_id=finding_type_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingTypeGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingTypeGetResponse]], ResultWrapper[FindingTypeGetResponse]),
        )


class FindingTypesResourceWithRawResponse:
    def __init__(self, finding_types: FindingTypesResource) -> None:
        self._finding_types = finding_types

        self.list = to_raw_response_wrapper(
            finding_types.list,
        )
        self.get = to_raw_response_wrapper(
            finding_types.get,
        )

    @cached_property
    def remediation_types(self) -> RemediationTypesResourceWithRawResponse:
        return RemediationTypesResourceWithRawResponse(self._finding_types.remediation_types)


class AsyncFindingTypesResourceWithRawResponse:
    def __init__(self, finding_types: AsyncFindingTypesResource) -> None:
        self._finding_types = finding_types

        self.list = async_to_raw_response_wrapper(
            finding_types.list,
        )
        self.get = async_to_raw_response_wrapper(
            finding_types.get,
        )

    @cached_property
    def remediation_types(self) -> AsyncRemediationTypesResourceWithRawResponse:
        return AsyncRemediationTypesResourceWithRawResponse(self._finding_types.remediation_types)


class FindingTypesResourceWithStreamingResponse:
    def __init__(self, finding_types: FindingTypesResource) -> None:
        self._finding_types = finding_types

        self.list = to_streamed_response_wrapper(
            finding_types.list,
        )
        self.get = to_streamed_response_wrapper(
            finding_types.get,
        )

    @cached_property
    def remediation_types(self) -> RemediationTypesResourceWithStreamingResponse:
        return RemediationTypesResourceWithStreamingResponse(self._finding_types.remediation_types)


class AsyncFindingTypesResourceWithStreamingResponse:
    def __init__(self, finding_types: AsyncFindingTypesResource) -> None:
        self._finding_types = finding_types

        self.list = async_to_streamed_response_wrapper(
            finding_types.list,
        )
        self.get = async_to_streamed_response_wrapper(
            finding_types.get,
        )

    @cached_property
    def remediation_types(self) -> AsyncRemediationTypesResourceWithStreamingResponse:
        return AsyncRemediationTypesResourceWithStreamingResponse(self._finding_types.remediation_types)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["QueryCreateParams"]


class QueryCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    alert_enabled: Required[bool]
    """Enable alerts for this query"""

    alert_rollup_enabled: Required[bool]
    """Enable alert rollup for this query"""

    name: Required[str]
    """Unique name for the saved query"""

    query_json: Required[str]
    """JSON string containing the query parameters"""

    rule_enabled: Required[bool]
    """Enable rule for this query"""

    rule_scope: str
    """Scope for the rule"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .findings import (
    FindingsResource,
    AsyncFindingsResource,
    FindingsResourceWithRawResponse,
    AsyncFindingsResourceWithRawResponse,
    FindingsResourceWithStreamingResponse,
    AsyncFindingsResourceWithStreamingResponse,
)
from .instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)

__all__ = [
    "InstancesResource",
    "AsyncInstancesResource",
    "InstancesResourceWithRawResponse",
    "AsyncInstancesResourceWithRawResponse",
    "InstancesResourceWithStreamingResponse",
    "AsyncInstancesResourceWithStreamingResponse",
    "FindingsResource",
    "AsyncFindingsResource",
    "FindingsResourceWithRawResponse",
    "AsyncFindingsResourceWithRawResponse",
    "FindingsResourceWithStreamingResponse",
    "AsyncFindingsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FindingTypeListParams"]


class FindingTypeListParams(TypedDict, total=False):
    account_id: Required[str]

    page: int
    """A page number within the paginated result set."""

    per_page: int
    """Number of results to return per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .category_edit_params import CategoryEditParams as CategoryEditParams
from .category_list_params import CategoryListParams as CategoryListParams
from .indicator_list_params import IndicatorListParams as IndicatorListParams
from .category_create_params import CategoryCreateParams as CategoryCreateParams
from .category_edit_response import CategoryEditResponse as CategoryEditResponse
from .category_list_response import CategoryListResponse as CategoryListResponse
from .indicator_list_response import IndicatorListResponse as IndicatorListResponse
from .category_create_response import CategoryCreateResponse as CategoryCreateResponse
from .category_delete_response import CategoryDeleteResponse as CategoryDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .routing import (
    RoutingResource,
    AsyncRoutingResource,
    RoutingResourceWithRawResponse,
    AsyncRoutingResourceWithRawResponse,
    RoutingResourceWithStreamingResponse,
    AsyncRoutingResourceWithStreamingResponse,
)
from .summary import (
    SummaryResource,
    AsyncSummaryResource,
    SummaryResourceWithRawResponse,
    AsyncSummaryResourceWithRawResponse,
    SummaryResourceWithStreamingResponse,
    AsyncSummaryResourceWithStreamingResponse,
)
from .timeseries_groups import (
    TimeseriesGroupsResource,
    AsyncTimeseriesGroupsResource,
    TimeseriesGroupsResourceWithRawResponse,
    AsyncTimeseriesGroupsResourceWithRawResponse,
    TimeseriesGroupsResourceWithStreamingResponse,
    AsyncTimeseriesGroupsResourceWithStreamingResponse,
)

__all__ = [
    "SummaryResource",
    "AsyncSummaryResource",
    "SummaryResourceWithRawResponse",
    "AsyncSummaryResourceWithRawResponse",
    "SummaryResourceWithStreamingResponse",
    "AsyncSummaryResourceWithStreamingResponse",
    "TimeseriesGroupsResource",
    "AsyncTimeseriesGroupsResource",
    "TimeseriesGroupsResourceWithRawResponse",
    "AsyncTimeseriesGroupsResourceWithRawResponse",
    "TimeseriesGroupsResourceWithStreamingResponse",
    "AsyncTimeseriesGroupsResourceWithStreamingResponse",
    "RoutingResource",
    "AsyncRoutingResource",
    "RoutingResourceWithRawResponse",
    "AsyncRoutingResourceWithRawResponse",
    "RoutingResourceWithStreamingResponse",
    "AsyncRoutingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .schema_data_param import SchemaDataParam

__all__ = ["DEXTestUpdateParams", "TargetPolicy"]


class DEXTestUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Unique identifier linked to an account."""

    data: Required[SchemaDataParam]
    """
    The configuration object which contains the details for the WARP client to
    conduct the test.
    """

    enabled: Required[bool]
    """Determines whether or not the test is active."""

    interval: Required[str]
    """How often the test will run."""

    name: Required[str]
    """The name of the DEX test. Must be unique."""

    description: str
    """Additional details about the test."""

    target_policies: Iterable[TargetPolicy]
    """DEX rules targeted by this test"""

    targeted: bool


class TargetPolicy(TypedDict, total=False):
    id: Required[str]
    """The id of the DEX rule."""

    default: bool
    """Whether the DEX rule is the account default."""

    name: str
    """The name of the DEX rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.cloudforce_one.threat_events.indicators import aggregate_list_params
from .....types.cloudforce_one.threat_events.indicators.aggregate_list_response import AggregateListResponse

__all__ = ["AggregateResource", "AsyncAggregateResource"]


class AggregateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AggregateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AggregateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AggregateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AggregateResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        aggregate_by: str,
        created_after: str | Omit = omit,
        created_before: str | Omit = omit,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        event_date_after: str | Omit = omit,
        event_date_before: str | Omit = omit,
        limit: float | Omit = omit,
        measure: Literal["indicators", "relationships"] | Omit = omit,
        tag_uuid: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AggregateListResponse:
        """
        Aggregate threat indicators by one or more columns (e.g., indicatorType, value)
        across datasets. Returns top-N groups ordered by count.

        Args:
          account_id: Account ID.

          aggregate_by: Column(s) to aggregate by - single column or comma-separated list (e.g.,
              'indicatorType', 'value', 'indicatorType,value')

          created_after: Filter indicators created after this date (ISO 8601 format, e.g., '2024-01-01')

          created_before: Filter indicators created before this date (ISO 8601 format, e.g., '2024-12-31')

          dataset_ids: Dataset ID(s) to filter by. Can be a single dataset ID or comma-separated list.
              If not provided, aggregates across all accessible datasets

          event_date_after: For measure=relationships: only count event links whose eventDate is on/after
              this date (ISO 8601). Use to bound 'top indicator' to recent activity.

          event_date_before: For measure=relationships: only count event links whose eventDate is on/before
              this date (ISO 8601).

          limit: Maximum number of aggregation results to return (1-100)

          measure: What to count per group: 'indicators' (catalog rows, default) or 'relationships'
              (linked events per indicator). Use 'relationships' for 'top indicator by event
              activity'.

          tag_uuid: Scope to indicators associated with this tag/actor UUID. Combine with
              measure=relationships for 'top indicator for an actor'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/indicators/aggregate", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "aggregate_by": aggregate_by,
                        "created_after": created_after,
                        "created_before": created_before,
                        "dataset_ids": dataset_ids,
                        "event_date_after": event_date_after,
                        "event_date_before": event_date_before,
                        "limit": limit,
                        "measure": measure,
                        "tag_uuid": tag_uuid,
                    },
                    aggregate_list_params.AggregateListParams,
                ),
            ),
            cast_to=AggregateListResponse,
        )


class AsyncAggregateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAggregateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAggregateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAggregateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAggregateResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        aggregate_by: str,
        created_after: str | Omit = omit,
        created_before: str | Omit = omit,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        event_date_after: str | Omit = omit,
        event_date_before: str | Omit = omit,
        limit: float | Omit = omit,
        measure: Literal["indicators", "relationships"] | Omit = omit,
        tag_uuid: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AggregateListResponse:
        """
        Aggregate threat indicators by one or more columns (e.g., indicatorType, value)
        across datasets. Returns top-N groups ordered by count.

        Args:
          account_id: Account ID.

          aggregate_by: Column(s) to aggregate by - single column or comma-separated list (e.g.,
              'indicatorType', 'value', 'indicatorType,value')

          created_after: Filter indicators created after this date (ISO 8601 format, e.g., '2024-01-01')

          created_before: Filter indicators created before this date (ISO 8601 format, e.g., '2024-12-31')

          dataset_ids: Dataset ID(s) to filter by. Can be a single dataset ID or comma-separated list.
              If not provided, aggregates across all accessible datasets

          event_date_after: For measure=relationships: only count event links whose eventDate is on/after
              this date (ISO 8601). Use to bound 'top indicator' to recent activity.

          event_date_before: For measure=relationships: only count event links whose eventDate is on/before
              this date (ISO 8601).

          limit: Maximum number of aggregation results to return (1-100)

          measure: What to count per group: 'indicators' (catalog rows, default) or 'relationships'
              (linked events per indicator). Use 'relationships' for 'top indicator by event
              activity'.

          tag_uuid: Scope to indicators associated with this tag/actor UUID. Combine with
              measure=relationships for 'top indicator for an actor'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/indicators/aggregate", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "aggregate_by": aggregate_by,
                        "created_after": created_after,
                        "created_before": created_before,
                        "dataset_ids": dataset_ids,
                        "event_date_after": event_date_after,
                        "event_date_before": event_date_before,
                        "limit": limit,
                        "measure": measure,
                        "tag_uuid": tag_uuid,
                    },
                    aggregate_list_params.AggregateListParams,
                ),
            ),
            cast_to=AggregateListResponse,
        )


class AggregateResourceWithRawResponse:
    def __init__(self, aggregate: AggregateResource) -> None:
        self._aggregate = aggregate

        self.list = to_raw_response_wrapper(
            aggregate.list,
        )


class AsyncAggregateResourceWithRawResponse:
    def __init__(self, aggregate: AsyncAggregateResource) -> None:
        self._aggregate = aggregate

        self.list = async_to_raw_response_wrapper(
            aggregate.list,
        )


class AggregateResourceWithStreamingResponse:
    def __init__(self, aggregate: AggregateResource) -> None:
        self._aggregate = aggregate

        self.list = to_streamed_response_wrapper(
            aggregate.list,
        )


class AsyncAggregateResourceWithStreamingResponse:
    def __init__(self, aggregate: AsyncAggregateResource) -> None:
        self._aggregate = aggregate

        self.list = async_to_streamed_response_wrapper(
            aggregate.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["ContentFindingTopNParams", "Filter"]


class ContentFindingTopNParams(TypedDict, total=False):
    account_id: Required[str]

    filters: Required[Iterable[Filter]]
    """Filters to apply.

    `findingType = content` is applied automatically for CASB data.
    """

    from_: Required[Annotated[Union[str, datetime], PropertyInfo(alias="from", format="iso8601")]]
    """Start of the query time range (inclusive). RFC3339."""

    n: Required[int]
    """Maximum number of integrations to return."""

    to: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """End of the query time range (exclusive). RFC3339."""


class Filter(TypedDict, total=False):
    """A filter to apply to the query."""

    name: Required[str]
    """Specifies the column name to filter on.

    Requires a valid column for the target dataset (e.g. `country`, `allowed`,
    `appId`).
    """

    op: Required[str]
    """Filter operator.

    Common values: `eq`, `neq`, `in`, `not_in`, `gt`, `lt`, `gte`, `lte`.
    """

    values: Required[SequenceNotStr[Union[str, bool, float]]]
    """Values to match against. Type depends on the column."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RelayUpdateResponse", "Config", "ConfigUpstreams", "ConfigUpstreamsUpstream"]


class ConfigUpstreamsUpstream(BaseModel):
    """A single upstream MOQT server publisher."""

    url: str
    """Upstream MOQT server publisher URL.

    Must be an absolute URL with a host and a scheme crique can dial: moqt:// (raw
    QUIC) or https:// (WebTransport). Validated on update (PUT); rejected
    with 21013.
    """


class ConfigUpstreams(BaseModel):
    """
    Upstreams are external MOQT server publishers that a relay falls back
    to when it has no local publisher for a requested namespace/track.
    """

    enabled: Optional[bool] = None

    upstreams: Optional[List[ConfigUpstreamsUpstream]] = None
    """Ordered list of upstream MOQT server publishers.

    Each entry is an object (not a bare string) so per-upstream configuration can be
    added in the future without another breaking change.
    """


class Config(BaseModel):
    upstreams: Optional[ConfigUpstreams] = None
    """
    Upstreams are external MOQT server publishers that a relay falls back to when it
    has no local publisher for a requested namespace/track.
    """


class RelayUpdateResponse(BaseModel):
    """Full relay details (no tokens)."""

    config: Config

    created: datetime

    modified: datetime

    name: str

    uid: str

    status: Optional[Literal["connected"]] = None
    """\"connected" when active, omitted otherwise."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SessionGetSessionTranscriptsParams"]


class SessionGetSessionTranscriptsParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    format: Literal["SRT", "VTT", "JSON", "CSV"]
    """Transcript file format to fetch."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .idp_federation_grant import IdPFederationGrant

__all__ = ["IdPFederationGrantListResponse"]

IdPFederationGrantListResponse: TypeAlias = List[IdPFederationGrant]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ......_types import SequenceNotStr
from ......_utils import PropertyInfo

__all__ = ["JobExportParams", "Order"]


class JobExportParams(TypedDict, total=False):
    account_id: Required[str]

    integration_id: SequenceNotStr[str]
    """Filter by multiple integration IDs."""

    max_updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view remediation jobs updated on or before this datetime.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    min_updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view remediation jobs updated on or after this datetime.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    orders: Iterable[Order]
    """Ordering specifications for the export."""

    search: str
    """A search term."""

    status: List[Literal["pending", "processing", "completed", "failed", "validating"]]
    """Filter by remediation job status."""


class Order(TypedDict, total=False):
    """Order specification for remediation jobs exports."""

    direction: Required[Literal["asc", "desc"]]
    """Sort direction."""

    name: Required[
        Literal["asset_name", "finding_type_name", "integration_name", "status", "last_updated_at", "affliction_date"]
    ]
    """Which field to use when ordering the remediation jobs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .analytics_query import (
    AnalyticsQueryResource,
    AsyncAnalyticsQueryResource,
    AnalyticsQueryResourceWithRawResponse,
    AsyncAnalyticsQueryResourceWithRawResponse,
    AnalyticsQueryResourceWithStreamingResponse,
    AsyncAnalyticsQueryResourceWithStreamingResponse,
)

__all__ = [
    "AnalyticsQueryResource",
    "AsyncAnalyticsQueryResource",
    "AnalyticsQueryResourceWithRawResponse",
    "AsyncAnalyticsQueryResourceWithRawResponse",
    "AnalyticsQueryResourceWithStreamingResponse",
    "AsyncAnalyticsQueryResourceWithStreamingResponse",
]
